package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperAction;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.Crop;
import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.model.Station;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.scraper.ScraperUtil.findOption;
import static com.argotec.scraper.ScraperUtil.selectOption;
import static com.argotec.siar.scraper.ScraperVariables.*;

@Slf4j
public class ScraperStations implements ScraperDataSource<Station> {

    private final List<Crop> existingCrops;
    private final String community;
    private Province province;
    private IrrigationCommunity irrigationCommunity;

    public ScraperStations(List<Crop> existingCrops, String community, Province province) {
        this.existingCrops = existingCrops;
        this.community = community;
        this.province = province;
    }

    public ScraperStations(List<Crop> existingCrops, String community, IrrigationCommunity irrigationCommunity) {
        this.existingCrops = existingCrops;
        this.community = community;
        this.irrigationCommunity = irrigationCommunity;
    }

    @Override
    public List<Station> scrapeAll() {
        String name;
        ScraperAction subAction;
        if (province != null) {
            name = province.getName();
            subAction = selectProvinceAction(province.getName());
        } else {
            name = irrigationCommunity.getName();
            subAction = selectIrrigationCommunityAction(irrigationCommunity.getName());
        }

        log.info("Fetching stations of {} using WebScrapper", name);

        try (Scraper scraper = source(ScrapingUrl.getUrlNn)) {
            scraper.doAction(selectAuCommunityAction(community))
                    .doAction(subAction);

            List<String> stationNames = findSelect(scraper)
                    .getOptions()
                    .stream()
                    .map(WebElement::getText)
                    .collect(Collectors.toList());

            return stationNames
                    .stream()
                    .map(stationName -> parseStationFromWeb(scraper, stationName))
                    .collect(Collectors.toList());
        }
    }

    public Station scrapeOne(String name) {
        log.info("Fetching station {} of {} using WebScrapper", name, province.getName());
        try (Scraper scraper = source(ScrapingUrl.getUrlNn)) {
            scraper.doAction(selectAuCommunityAction(community))
                    .doAction(selectProvinceAction(province.getName()));

            WebElement optionEl = findOption(findSelect(scraper), name);
            if (optionEl != null) {
                return parseStationFromWeb(scraper, optionEl.getText());
            }
            return null;
        }
    }

    private ISelect findSelect(Scraper scraper) {
        return scraper.findRootElement(By.id("ContentPlaceHolder1_cboEstaciones")).toSelectElement();
    }

    private Station parseStationFromWeb(Scraper scraper, String stationName) {
        log.info("Parsing station {}", stationName);
        return new Station(stationName, province, irrigationCommunity);
        // TODO enable crops?
        /*Station station = new Station(stationName, province, irrigationCommunity);
        scraper.doAction(selectStationAction(stationName));
        scraper.waitSeconds(2);
        station.setCrops(parseSelectedStationCropsFromWeb(scraper, station));
        return station;*/
    }

    private Set<Crop> parseSelectedStationCropsFromWeb(Scraper scraper, Station station) {
        return scraper.findRootElement(By.id("ContentPlaceHolder1_cboCultivo"))
                .toSelectElement()
                .getOptions()
                .stream()
                .map(op -> {
                    String cropName = op.getText();
                    log.info("Parsing station {} crop {}", station.getName(), cropName);
                    Crop crop = getCropFromList(cropName);
                    if (crop == null) {
                        log.info("Creating a new crop {}", cropName);
                        crop = new Crop();
                        crop.setName(cropName);
                        existingCrops.add(crop);
                    }
                    return crop;
                })
                .collect(Collectors.toSet());
    }

    private Crop getCropFromList(String cropName) {
        return existingCrops.stream().filter(c -> c.getName().equals(cropName)).findFirst().orElse(null);
    }

    public static ScraperAction addStation(String station) {
        return scraper -> {
            String onErrorMessage = "ERROR: " + station + " - Incorrectly introduced station";
            scraper.doAction(selectOption("DropDownListEstacion", station, onErrorMessage));

            try {
                scraper.findRootElement(By.id("ContentPlaceHolder1_ButtonAgregar"))
                        .click()
                        .and()
                        .waitSeconds(2);
            } catch (Exception e) {
                System.out.println("ERROR: " + station + " - Station incorrectly added");
            }
        };
    }
}
