package com.argotec.siar.scraper;

import com.argotec.scraper.Function;
import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperAction;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.Crop;
import com.argotec.siar.model.Station;
import com.argotec.siar.model.Variable;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;

import java.time.LocalDate;
import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.scraper.ScraperUtil.selectOption;
import static com.argotec.siar.scraper.SiarScraperUtil.setEndDate;
import static com.argotec.siar.scraper.SiarScraperUtil.setStartDate;

@Slf4j
public class ScraperVariables implements ScraperDataSource<Variable> {

    private final Station station;
    private final Crop crop;
    private final LocalDate startDate;
    private final LocalDate endDate;

    public ScraperVariables(Station station, Crop crop, LocalDate startDate, LocalDate endDate) {
        this.station = station;
        this.crop = crop;
        this.startDate = startDate;
        this.endDate = endDate;
    }

    @Override
    public List<Variable> scrapeAll() {
        try (Scraper scraper = source(ScrapingUrl.getUrlNn)) {
            scraper.doAction(selectAuCommunityAction(station.getProvince().getAutonomousCommunity().getName()))
                    .doAction(selectProvinceAction(station.getProvince().getName()))
                    .doAction(selectStationAction(station.getName()))
                    .doAction(selectCropAction(crop.getName()))
                    .doAction(setStartDate(startDate))
                    .doAction(setEndDate(endDate));

            scraper.findRootElement(By.id("bntEjecutarConsultaNH"))
                    .click();

            scraper.waitSeconds(2)
                    .switchTo()
                    .waitSeconds(2);

            return toVariables(Function.createTable(scraper.getWebDriver()));
        }
    }

    private List<Variable> toVariables(List<List<String>> table) {
        return table.stream()
                .skip(1)
                .map(row -> new Variable(crop.getId(), row))
                .collect(Collectors.toList());
    }

    public static ScraperAction selectAuCommunityAction(String auCommunityName) {
        String onErrorMessage = "ERROR: " + auCommunityName + " - Incorrectly introduced AACC";
        return selectOption("ContentPlaceHolder1_cboCCAA", auCommunityName, onErrorMessage);
    }

    public static ScraperAction selectProvinceAction(String provinceName) {
        String onErrorMessage = "ERROR: " + provinceName + " - Incorrectly introduced province";
        return scraper -> {
            scraper.findRootElement(By.id("ContentPlaceHolder1_rbProvincia")).click();
            scraper.waitSeconds(2);
            scraper.doAction(selectOption("ContentPlaceHolder1_cboProvincia", provinceName, onErrorMessage));
        };
    }

    public static ScraperAction selectIrrigationCommunityAction(String irrigationCommunityName) {
        String onErrorMessage = "ERROR: " + irrigationCommunityName + " - Incorrectly introduced irrigation community";
        return scraper -> {
            scraper.findRootElement(By.id("ContentPlaceHolder1_rbComRegante")).click();
            scraper.waitSeconds(2);
            scraper.doAction(selectOption("ContentPlaceHolder1_cboCR", irrigationCommunityName, onErrorMessage));
        };
    }

    public static ScraperAction selectStationAction(String stationName) {
        String onErrorMessage = "ERROR: " + stationName + " - Incorrectly introduced station";
        return selectOption("ContentPlaceHolder1_cboEstaciones", stationName, onErrorMessage);
    }

    public static ScraperAction selectCropAction(String cropName) {
        String onErrorMessage = "ERROR: " + cropName + " - Incorrectly introduced crop";
        return selectOption("ContentPlaceHolder1_cboCultivo", cropName, onErrorMessage);
    }
}
